@@ -52,6 +52,8 @@ struct _NMDevicePrivate
 	char *			driver;
 	gboolean			removed;
 
+	struct ether_addr	hw_addr;
+
 	gboolean			link_active;
 	guint32			ip4_address;
 	struct in6_addr	ip6_address;
@@ -85,14 +87,22 @@ static NMDeviceType
 discover_device_type (LibHalContext *ctx, const char *udi)
 {
 	char * category = NULL;
+	NMDeviceType type = DEVICE_TYPE_UNKNOWN;
 
 	if (libhal_device_property_exists (ctx, udi, "info.category", NULL))
 		category = libhal_device_get_property_string(ctx, udi, "info.category", NULL);
-	if (category && (!strcmp (category, "net.80211")))
-		return DEVICE_TYPE_802_11_WIRELESS;
-	else if (category && (!strcmp (category, "net.80203")))
-		return DEVICE_TYPE_802_3_ETHERNET;
-	return DEVICE_TYPE_UNKNOWN;
+
+	if (category)
+	{
+		if (!strcmp (category, "net.80211"))
+			type = DEVICE_TYPE_802_11_WIRELESS;
+		else if (!strcmp (category, "net.80203"))
+			type = DEVICE_TYPE_802_3_ETHERNET;
+
+		libhal_free_string (category);
+	}
+
+	return type;
 }
 
 /*
@@ -175,12 +185,7 @@ nm_device_new (const char *iface,
 	nm_device_bring_up_wait (dev, FALSE);
 
 	nm_device_update_ip4_address (dev);
-
-	/* Update the device's hardware address */
-	if (nm_device_is_802_3_ethernet (dev))
-		nm_device_802_3_ethernet_set_address (NM_DEVICE_802_3_ETHERNET (dev));
-	else if (nm_device_is_802_11_wireless (dev))
-		nm_device_802_11_wireless_set_address (NM_DEVICE_802_11_WIRELESS (dev));
+	nm_device_update_hw_address (dev);
 
 	/* Grab IP config data for this device from the system configuration files */
 	dev->priv->system_config_data = nm_system_device_get_system_config (dev, app_data);
@@ -247,6 +252,8 @@ nm_device_init (NMDevice * self)
 	self->priv->loop = NULL;
 	self->priv->worker = NULL;
 	self->priv->worker_started = FALSE;
+
+	memset (&(self->priv->hw_addr), 0, sizeof (struct ether_addr));
 }
 
 static guint32
@@ -382,6 +389,43 @@ nm_get_device_by_iface (NMData *data,
 }
 
 
+/*
+ * nm_get_device_by_iface_locked
+ *
+ * Search through the device list for a device with a given iface.
+ * NOTE: refs the device, caller must unref when done.
+ *
+ */
+NMDevice *
+nm_get_device_by_iface_locked (NMData *data,
+                               const char *iface)
+{
+	GSList *	elt;
+	NMDevice *dev = NULL;
+	
+	g_return_val_if_fail (data  != NULL, NULL);
+	g_return_val_if_fail (iface != NULL, NULL);
+
+	nm_lock_mutex (data->dev_list_mutex, __func__);
+	for (elt = data->dev_list; elt; elt = g_slist_next (elt))
+	{
+		NMDevice	*tmp_dev = NULL;
+		if ((tmp_dev = NM_DEVICE (elt->data)))
+		{
+			if (nm_null_safe_strcmp (nm_device_get_iface (tmp_dev), iface) == 0)
+			{
+				g_object_ref (G_OBJECT (tmp_dev));
+				dev = tmp_dev;
+				break;
+			}
+		}
+	}
+	nm_unlock_mutex (data->dev_list_mutex, __FUNCTION__);
+
+	return dev;
+}
+
+
 /*
  * Get/set functions for UDI
  */
@@ -545,45 +589,46 @@ nm_device_set_active_link (NMDevice *self,
 	app_data = self->priv->app_data;
 	req = nm_device_get_act_request (self);
 
-	if (self->priv->link_active != link_active)
+	if (self->priv->link_active == link_active)
+		return;
+
+	self->priv->link_active = link_active;
+
+	/* Deactivate a currently active device */
+	if (!link_active && req)
+		nm_policy_schedule_device_change_check (app_data);
+	else if (link_active && !req)
 	{
-		self->priv->link_active = link_active;
+		NMDevice *	act_dev = nm_get_active_device (app_data);
+		NMActRequest *	act_dev_req = act_dev ? nm_device_get_act_request (act_dev) : NULL;
 
-		/* Deactivate a currently active device */
-		if (!link_active && req)
-			nm_policy_schedule_device_change_check (app_data);
-		else if (link_active && !req)
+		/* Should we switch to this device now that it has a link?
+		 *
+		 * Only auto-switch for wired devices, AND...
+		 *
+		 * only switch to fully-supported devices, since ones that don't have carrier detection
+		 * capability usually report the carrier as "always on" even if its not really on.  User
+		 * must manually choose semi-supported devices.
+		 *
+		 */
+		if (nm_device_is_802_3_ethernet (self) && (nm_device_get_capabilities (self) & NM_DEVICE_CAP_CARRIER_DETECT))
 		{
-			NMDevice *	act_dev = nm_get_active_device (app_data);
-			NMActRequest *	act_dev_req = act_dev ? nm_device_get_act_request (act_dev) : NULL;
+			gboolean do_switch = act_dev ? FALSE : TRUE;	/* If no currently active device, switch to this one */
 
-			/* Should we switch to this device now that it has a link?
-			 *
-			 * Only auto-switch for wired devices, AND...
-			 *
-			 * only switch to fully-supported devices, since ones that don't have carrier detection
-			 * capability usually report the carrier as "always on" even if its not really on.  User
-			 * must manually choose semi-supported devices.
-			 *
-			 */
-			if (nm_device_is_802_3_ethernet (self) && (nm_device_get_capabilities (self) & NM_DEVICE_CAP_CARRIER_DETECT))
+			/* If active device is wireless, switch to this one */
+			if (act_dev && nm_device_is_802_11_wireless (act_dev) && act_dev_req && !nm_act_request_get_user_requested (act_dev_req))
+				do_switch = TRUE;
+
+			/* FIXME: Why is this activation request created here and never used? */
+			/* if (do_switch && (act_req = nm_act_request_new (app_data, self, NULL, TRUE))) */
+			if (do_switch)
 			{
-				gboolean 		do_switch = act_dev ? FALSE : TRUE;	/* If no currently active device, switch to this one */
-				NMActRequest *	act_req;
-
-				/* If active device is wireless, switch to this one */
-				if (act_dev && nm_device_is_802_11_wireless (act_dev) && act_dev_req && !nm_act_request_get_user_requested (act_dev_req))
-					do_switch = TRUE;
-
-				if (do_switch && (act_req = nm_act_request_new (app_data, self, NULL, TRUE)))
-				{
-					nm_info ("Will activate wired connection '%s' because it now has a link.", nm_device_get_iface (self));
-					nm_policy_schedule_device_change_check (app_data);
-				}
+				nm_info ("Will activate wired connection '%s' because it now has a link.", nm_device_get_iface (self));
+				nm_policy_schedule_device_change_check (app_data);
 			}
 		}
-		nm_dbus_schedule_device_status_change_signal	(app_data, self, NULL, link_active ? DEVICE_CARRIER_ON : DEVICE_CARRIER_OFF);
 	}
+	nm_dbus_schedule_device_status_change_signal (app_data, self, NULL, link_active ? DEVICE_CARRIER_ON : DEVICE_CARRIER_OFF);
 }
 
 
@@ -669,6 +714,7 @@ nm_device_activate_stage1_device_prepare (NMActRequest *req)
 	nm_device_activate_schedule_stage2_device_config (req);
 
 out:
+	nm_act_request_unref (req);
 	nm_info ("Activation (%s) Stage 1 of 5 (Device Prepare) complete.", iface);
 	return FALSE;
 }
@@ -692,6 +738,7 @@ nm_device_activate_schedule_stage1_device_prepare (NMActRequest *req)
 	g_assert (self);
 
 	nm_act_request_set_stage (req, NM_ACT_STAGE_DEVICE_PREPARE);
+	nm_act_request_ref (req);
 	nm_info ("Activation (%s) Stage 1 of 5 (Device Prepare) scheduled...", nm_device_get_iface (self));
 
 	source = g_idle_source_new ();
@@ -765,6 +812,7 @@ nm_device_activate_stage2_device_config (NMActRequest *req)
 	nm_device_activate_schedule_stage3_ip_config_start (req);
 
 out:
+	nm_act_request_unref (req);
 	nm_info ("Activation (%s) Stage 2 of 5 (Device Configure) complete.", iface);
 	return FALSE;
 }
@@ -788,6 +836,7 @@ nm_device_activate_schedule_stage2_device_config (NMActRequest *req)
 	g_assert (self);
 
 	nm_act_request_set_stage (req, NM_ACT_STAGE_DEVICE_CONFIG);
+	nm_act_request_ref (req);
 
 	source = g_idle_source_new ();
 	g_source_set_callback (source, (GSourceFunc) nm_device_activate_stage2_device_config, req, NULL);
@@ -873,6 +922,7 @@ nm_device_activate_stage3_ip_config_start (NMActRequest *req)
 
 out:
 	nm_info ("Activation (%s) Stage 3 of 5 (IP Configure Start) complete.", iface);
+	nm_act_request_unref (req);
 	return FALSE;
 }
 
@@ -894,6 +944,7 @@ nm_device_activate_schedule_stage3_ip_config_start (NMActRequest *req)
 	g_assert (self);
 
 	nm_act_request_set_stage (req, NM_ACT_STAGE_IP_CONFIG_START);
+	nm_act_request_ref (req);
 
 	source = g_idle_source_new ();
 	g_source_set_callback (source, (GSourceFunc) nm_device_activate_stage3_ip_config_start, req, NULL);
@@ -1014,9 +1065,11 @@ nm_device_activate_stage4_ip_config_get (NMActRequest *req)
 		goto out;
 
 	nm_act_request_set_ip4_config (req, ip4_config);
+	nm_ip4_config_unref (ip4_config);
 	nm_device_activate_schedule_stage5_ip_config_commit (req);
 
 out:
+	nm_act_request_unref (req);
 	nm_info ("Activation (%s) Stage 4 of 5 (IP Configure Get) complete.", nm_device_get_iface (self));
 	return FALSE;
 }
@@ -1040,6 +1093,7 @@ nm_device_activate_schedule_stage4_ip_config_get (NMActRequest *req)
 	g_assert (self);
 
 	nm_act_request_set_stage (req, NM_ACT_STAGE_IP_CONFIG_GET);
+	nm_act_request_ref (req);
 	nm_info ("Activation (%s) Stage 4 of 5 (IP Configure Get) scheduled...",
 			nm_device_get_iface (self));
 
@@ -1109,6 +1163,7 @@ nm_device_activate_stage4_ip_config_timeout (NMActRequest *req)
 	g_assert (ip4_config);
 
 	nm_act_request_set_ip4_config (req, ip4_config);
+	nm_ip4_config_unref (ip4_config);
 	nm_device_activate_schedule_stage5_ip_config_commit (req);
 
 out:
@@ -1191,6 +1246,7 @@ nm_device_activate_stage5_ip_config_commit (NMActRequest *req)
 		nm_policy_schedule_activation_failed (req);
 
 out:
+	nm_act_request_unref (req);
 	nm_info ("Activation (%s) Stage 5 of 5 (IP Configure Commit) complete.",
 			nm_device_get_iface (self));
 	return FALSE;
@@ -1214,6 +1270,7 @@ nm_device_activate_schedule_stage5_ip_config_commit (NMActRequest *req)
 	g_assert (self);
 
 	nm_act_request_set_stage (req, NM_ACT_STAGE_IP_CONFIG_COMMIT);
+	nm_act_request_ref (req);
 
 	source = g_idle_source_new ();
 	g_source_set_callback (source, (GSourceFunc) nm_device_activate_stage5_ip_config_commit, req, NULL);
@@ -1668,6 +1725,42 @@ nm_device_update_ip4_address (NMDevice *self)
 }
 
 
+void
+nm_device_get_hw_address (NMDevice *self,
+                          struct ether_addr *addr)
+{
+	g_return_if_fail (self != NULL);
+	g_return_if_fail (addr != NULL);
+
+	memcpy (addr, &(self->priv->hw_addr), sizeof (struct ether_addr));
+}
+
+void
+nm_device_update_hw_address (NMDevice *self)
+{
+	struct ifreq req;
+	NMSock *sk;
+	int ret;
+
+	g_return_if_fail (self != NULL);
+
+	sk = nm_dev_sock_open (self, DEV_GENERAL, __FUNCTION__, NULL);
+	if (!sk)
+		return;
+	memset (&req, 0, sizeof (struct ifreq));
+	strncpy (req.ifr_name, nm_device_get_iface (self), sizeof (req.ifr_name) - 1);
+
+	ret = ioctl (nm_dev_sock_get_fd (sk), SIOCGIFHWADDR, &req);
+	if (ret)
+		goto out;
+
+	memcpy (&(self->priv->hw_addr), &(req.ifr_hwaddr.sa_data), sizeof (struct ether_addr));
+
+out:
+	nm_dev_sock_close (sk);
+}
+
+
 /*
  * nm_device_set_up_down
  *
@@ -1686,10 +1779,7 @@ nm_device_set_up_down (NMDevice *self,
 	 * Make sure that we have a valid MAC address, some cards reload firmware when they
 	 * are brought up.
 	 */
-	if (nm_device_is_802_3_ethernet (self))
-		nm_device_802_3_ethernet_set_address (NM_DEVICE_802_3_ETHERNET (self));
-	else if (nm_device_is_802_11_wireless (self))
-		nm_device_802_11_wireless_set_address (NM_DEVICE_802_11_WIRELESS (self));
+	nm_device_update_hw_address (self);
 }
 
 
