@@ -347,8 +347,6 @@ static void nm_dbus_vpn_update_one_connection_cb (DBusPendingCall *pcall, void *
 	g_return_if_fail (cb_data->data != NULL);
 	g_return_if_fail (cb_data->data->vpn_manager != NULL);
 
-	dbus_pending_call_ref (pcall);
-
 	if (!dbus_pending_call_get_completed (pcall))
 		goto out;
 
@@ -418,8 +416,6 @@ static void nm_dbus_vpn_connections_update_cb (DBusPendingCall *pcall, void *use
 	g_return_if_fail (pcall);
 	g_return_if_fail (data != NULL);
 
-	dbus_pending_call_ref (pcall);
-
 	if (!dbus_pending_call_get_completed (pcall))
 		goto out;
 
@@ -540,10 +536,7 @@ static gboolean nm_dbus_vpn_connections_update_from_nmi (NMData *data)
 	dbus_connection_send_with_reply (data->dbus_connection, message, &pcall, -1);
 	dbus_message_unref (message);
 	if (pcall)
-	{
 		dbus_pending_call_set_notify (pcall, nm_dbus_vpn_connections_update_cb, data, NULL);
-		dbus_pending_call_block (pcall);
-	}
 
 	return FALSE;
 }
